use anyhow::Result;
use clap::Parser;
use sb_ruleset_sync::cli::Args;
use tracing::error;
use tracing_subscriber::fmt::time::LocalTime;

#[tokio::main]
async fn main() -> Result<()> {
    // 1. Parse command-line arguments first to get the verbosity level.
    let args = Args::parse();

    // 2. Set up the tracing subscriber.
    // This configures a logger that prints to the console.
    // The verbosity is controlled by the `-v` flag.
    let log_level = match args.verbose {
        0 => "info",  // Default level
        1 => "debug", // -v
        _ => "trace", // -vv or more
    };

    // We construct a filter that applies the log level to our crate,
    // but keeps other crates at a higher level (e.g., info) to reduce noise.
    let filter = format!("sb_ruleset_sync={}", log_level);

    tracing_subscriber::fmt()
        .with_env_filter(filter)
        .with_writer(std::io::stderr) // Log to stderr, common for daemons/cron
        .with_timer(LocalTime::rfc_3339())
        .init();

    // 3. Execute the main application logic.
    if let Err(e) = sb_ruleset_sync::run(args).await {
        // Log the final error in a structured way before exiting.
        error!(error = ?e, "Application failed");
        std::process::exit(1);
    }

    Ok(())
}
